@@ -39,7 +39,7 @@ struct binfmt_misc_info {
 
 LIST_HEAD(binfmt_misc_list);
 
-static int binfmt_misc_parse(struct mount_info *pm)
+static int binfmt_misc_parse_or_collect(struct mount_info *pm)
 {
 	opts.has_binfmt_misc = true;
 	return 0;
@@ -375,7 +375,7 @@ int collect_binfmt_misc(void)
 #else
 #define binfmt_misc_dump	NULL
 #define binfmt_misc_restore	NULL
-#define binfmt_misc_parse	NULL
+#define binfmt_misc_parse_or_collect NULL
 #endif
 
 static int tmpfs_dump(struct mount_info *pm)
@@ -407,7 +407,7 @@ static int tmpfs_dump(struct mount_info *pm)
 	sprintf(tmpfs_path, "/proc/self/fd/%d", fd);
 
 	if (root_ns_mask & CLONE_NEWUSER)
-		userns_pid = root_item->pid.real;
+		userns_pid = root_item->pid->real;
 
 	ret = cr_system_userns(-1, img_raw_fd(img), -1, "tar", (char *[])
 			{ "tar", "--create",
@@ -574,6 +574,17 @@ static int tracefs_parse(struct mount_info *pm)
 	return 1;
 }
 
+static bool cgroup_sb_equal(struct mount_info *a, struct mount_info *b)
+{
+	if (a->private && b->private &&
+			strcmp(a->private, b->private))
+		return false;
+	if (strcmp(a->options, b->options))
+		return false;
+
+	return true;
+}
+
 static int cgroup_parse(struct mount_info *pm)
 {
 	if (!(root_ns_mask & CLONE_NEWCGROUP))
@@ -590,6 +601,40 @@ static int cgroup_parse(struct mount_info *pm)
 	return 0;
 }
 
+static bool btrfs_sb_equal(struct mount_info *a, struct mount_info *b)
+{
+	/* There is a btrfs bug where it doesn't emit subvol= correctly when
+	 * files are bind mounted, so let's ignore it for now.
+	 * https://marc.info/?l=linux-btrfs&m=145857372803614&w=2
+	 */
+
+	char *posa = strstr(a->options, "subvol="), *posb = strstr(b->options, "subvol=");
+	bool equal;
+
+	if (!posa || !posb) {
+		pr_err("invalid btrfs options, no subvol argument");
+		return false;
+	}
+
+	*posa = *posb = 0;
+	equal = !strcmp(a->options, b->options);
+	*posa = *posb = 's';
+
+	if (!equal)
+		return false;
+
+	posa = strchr(posa, ',');
+	posb = strchr(posb, ',');
+
+	if ((posa && !posb) || (!posa && posb))
+		return false;
+
+	if (posa && strcmp(posa, posb))
+		return false;
+
+	return true;
+}
+
 static int dump_empty_fs(struct mount_info *pm)
 {
 	int fd, ret = -1;
@@ -639,7 +684,8 @@ static struct fstype fstypes[] = {
 		.restore = devtmpfs_restore,
 	}, {
 		.name = "binfmt_misc",
-		.parse = binfmt_misc_parse,
+		.parse = binfmt_misc_parse_or_collect,
+		.collect = binfmt_misc_parse_or_collect,
 		.code = FSTYPE__BINFMT_MISC,
 		.dump = binfmt_misc_dump,
 		.restore = binfmt_misc_restore,
@@ -658,6 +704,7 @@ static struct fstype fstypes[] = {
 	}, {
 		.name = "btrfs",
 		.code = FSTYPE__UNSUPPORTED,
+		.sb_equal = btrfs_sb_equal,
 	}, {
 		.name = "pstore",
 		.dump = dump_empty_fs,
@@ -685,6 +732,7 @@ static struct fstype fstypes[] = {
 		.name = "cgroup",
 		.code = FSTYPE__CGROUP,
 		.parse = cgroup_parse,
+		.sb_equal = cgroup_sb_equal,
 	}, {
 		.name = "aufs",
 		.code = FSTYPE__AUFS,
