@@ -16,7 +16,7 @@
 
 #include "image.h"
 #include "util-pie.h"
-#include "lock.h"
+#include "common/lock.h"
 
 #include "images/vma.pb-c.h"
 #include "images/tty.pb-c.h"
