@@ -4,7 +4,6 @@
 #include <linux/types.h>
 #include <sys/ptrace.h>
 
-#include "config.h"
 #include "proc_parse.h"
 
 /* some constants for ptrace */
@@ -31,14 +30,6 @@
 #define PTRACE_PEEKSIGINFO_SHARED       (1 << 0)
 #endif
 
-#ifndef CONFIG_HAS_PTRACE_PEEKSIGINFO
-struct ptrace_peeksiginfo_args {
-        __u64 off;	/* from which siginfo to start */
-        __u32 flags;
-        __u32 nr;	/* how may siginfos to take */
-};
-#endif
-
 #ifndef PTRACE_GETREGSET
 # define PTRACE_GETREGSET	0x4204
 # define PTRACE_SETREGSET	0x4205
